using System.Drawing;
using System.ComponentModel;
using System.Windows.Forms;
using System.IO;
using System.Text;

using Lesnikowski.Mail;
using Lesnikowski.Client;
using Lesnikowski.Mail.Headers;
using Lesnikowski.Mail.Headers.Constants;

public partial class MainForm : Form
{
    private ISimpleMailMessage _simpleMail;
    private IContainer components;

    public MainForm()
    {
        InitializeComponent();
        this.mailPanel.Dock = DockStyle.Fill;
        this.pictureBox.Dock = DockStyle.Fill;
        this.textBox.Dock = DockStyle.Fill;
    }

    private void selectFolderMenuItem_Click(object sender, System.EventArgs e)
    {
        if (this.folderBrowserDialog.ShowDialog() == DialogResult.OK)
        {
            //read all *.eml files in this folder.
            DirectoryInfo di = new DirectoryInfo(this.folderBrowserDialog.SelectedPath);
            FileInfo[] fiArray = di.GetFiles("*.eml");
            this.mailListView.BeginUpdate();
            this.mailListView.Items.Clear();
            ListViewItem item;
            for (int i = 0; i < fiArray.Length; i++)
            {
                item = new ListViewItem(fiArray[i].Name);
                item.Tag = fiArray[i];
                this.mailListView.Items.Add(item);
            }
            this.mailListView.EndUpdate();
        }
    }

    static void ProcessMime(MimeBase mime, TreeNode root)
    {
        string title = mime.ContentType == null ? string.Empty : mime.ContentType.ToString();

        TreeNode newRoot = root.Nodes.Add(title);
        newRoot.Tag = mime;

        if (mime is MimeMultipart)
        {
            MimeMultipart m = (MimeMultipart)mime;
            foreach (MimeBase part in m.Parts)
                ProcessMime(part, newRoot);
        }
    }

    private void ProcessMessage()
    {
        subject.Text = from.Text = "";
        this.toListBox.Items.Clear();
        this.ccListBox.Items.Clear();


        if (_simpleMail.Subject != null)
            this.subject.Text = _simpleMail.Subject;

        this.from.Text = _simpleMail.From.ToString();

        foreach (MailBox m in _simpleMail.To)
            this.toListBox.Items.Add(m.ToString());

        foreach (MailBox m in _simpleMail.Cc)
            this.ccListBox.Items.Add(m.ToString());

        this.mimeTreeView.Nodes.Clear();
        TreeNode root = this.mimeTreeView.Nodes.Add("root");
        ProcessMime(this._simpleMail.Document.Root, root);
        this.mimeTreeView.ExpandAll();
    }

    private void mailListView_SelectedIndexChanged(object sender, System.EventArgs e)
    {
        if (this.mailListView.SelectedItems.Count == 0)
            return;

        ListViewItem item = this.mailListView.SelectedItems[0];
        if (item.Tag.GetType() == typeof(FileInfo))
        {

            FileStream stream = ((FileInfo)item.Tag).OpenRead();
            StreamReader sr = new StreamReader(stream, Encoding.Default);
            try
            {
                string eml = sr.ReadToEnd();
                SimpleMailMessageBuilder builder = new SimpleMailMessageBuilder();
                this._simpleMail = builder.CreateFromEml(eml);
            }
            catch (MailException ex)
            {
                MessageBox.Show(ex.ToString());
                return;
            }
        }
        else
            _simpleMail = (ISimpleMailMessage)item.Tag;

        ProcessMessage();
    }

    private void mimeTreeView_AfterSelect(object sender, TreeViewEventArgs e)
    {
        if (this.pictureBox.Image != null)
        {
            this.pictureBox.Image.Dispose();
            this.pictureBox.Image = null;
        }

        if (e.Node.Parent == null)	//mail's info
        {
            this.mailPanel.BringToFront();
            return;
        }

        MimeBase mime = (MimeBase)e.Node.Tag;

        if (mime.ContentType == null ||
            mime.ContentType.MimeType == MimeType.Text ||
            mime.ContentType.MimeType == MimeType.Message)
        {
            MimeText mimeText = (MimeText)mime;
            this.textBox.BringToFront();
            this.textBox.Text = mimeText.Text;
        }
        else if (mime.ContentType.MimeType == MimeType.Image)
        {
            MimeData mimeData = (MimeData)mime;
            this.pictureBox.BringToFront();
            this.pictureBox.Image = Image.FromStream(mimeData.GetMemoryStream());
        }
        else
        {
            this.textBox.BringToFront();
            this.textBox.Text = mime.ToString();
        }
    }

    private void saveButton_Click(object sender, System.EventArgs e)
    {
        if (this.mimeTreeView.SelectedNode == null)
            return;

        if (this.mimeTreeView.SelectedNode.Tag is MimeData)
        {
            MimeData m = (MimeData)(this.mimeTreeView.SelectedNode.Tag);

            SaveFileDialog d = new SaveFileDialog();
            d.FileName = m.FileName;
            if (d.ShowDialog() == DialogResult.OK)
            {
                m.Save(d.FileName);
            }
        }
    }

    private void simpleViewButton_Click(object sender, System.EventArgs e)
    {
        if (_simpleMail == null)
            return;
        SimpleForm form = new SimpleForm();
        form.Bind(this._simpleMail);
        form.ShowDialog(this);
    }

    private void receiveMenuItem_Click(object sender, System.EventArgs e)
    {
        ServerSettings form = new ServerSettings();
        
        if (form.ShowDialog() == DialogResult.OK)
        {
            this.mailListView.Items.Clear();

            Pop3 pop3 = new Pop3();
            pop3.User = form.userTextBox.Text;
            pop3.Password = form.passwordTextBox.Text;
            try
            {
                pop3.Connect(form.serverTextBox.Text);
                if (pop3.HasTimeStamp == true && form.apopCheckBox.Checked == true)
                    pop3.APOPLogin();
                else
                    pop3.Login();
                pop3.GetAccountStat();

                ListViewItem item;
                SimpleMailMessageBuilder builder = new SimpleMailMessageBuilder();
                for (int i = 1; i <= pop3.MessageCount; i++)
                {
                    try
                    {
                        ISimpleMailMessage mail = builder.CreateFromEml(pop3.GetMessage(i));
                        item = new ListViewItem(mail.Subject);
                        item.Tag = mail;
                        this.mailListView.Items.Add(item);
                    }
                    catch (MailException ex)
                    {
                        MessageBox.Show(ex.Message);
                    }
                }
            }
            catch (ServerException ex)
            {
                MessageBox.Show(ex.Message);
            }
            finally
            {
                pop3.Close(false);
            }
        }
    }
};

